# -*- coding: utf-8 -*-
"""
Created on Thu Jan 16 14:28:58 2020

@author: mendez
"""

## The analytical form of the step function response of a system is
## y_S=1/5*(1-e^{-t}cos 2t-1/2 e^{-t}cos(2t)).

## Taks:
## 1. Plot this response.
## 2. Compute the impulse response h(t)
## 3. Compute the response of the continuous system (initially at rest) to u(t)=e^t u_S(t).
## 4. Compute the response of the discrete system for two sampling frequencies.
    

import numpy as np # Numpy is for scientific computing
import matplotlib.pyplot as plt  # Matplotlib is for plotting.
import shutil # To copy the figures in the Latex folder


## 1. Plot this response.
fs=100; dt=1/fs # Sampling frequency for the plot (very high!)
T=10 # Observation time of 3 seconds
t_k=np.arange(0,T,dt); # prepare the time axis
u_S=np.ones(t_k.shape); u_S[0]=0 # Step function
y_S=1/5-1/5*np.exp(-t_k)*(np.cos(2*t_k)+1/2*np.sin(2*t_k))

fig, ax = plt.subplots(figsize=(5, 3)) # This creates the figure
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
# Or you can plot it as streamlines
plt.plot(t_k,u_S,color='r',label='$u_{\mathcal{S}}(t)$')
plt.plot(t_k,y_S,'--',label='$y_{\mathcal{S}}(t)$')
ax.legend(fontsize=16)
plt.xlim(0,T)
plt.ylim(0,1.1)
plt.rc('text', usetex=True)      
plt.rc('font', family='serif')
plt.rc('xtick',labelsize=16)
plt.rc('ytick',labelsize=16)
ax.set_xlabel('$t[s]$',fontsize=16)
plt.tight_layout()
Name='Ex2_Step.pdf'
plt.savefig(Name, dpi=200) 
#shutil.copyfile(Name,'../Figures/'+Name) #  copy file to latex directory


## 2. Compute the impulse response h(t)

import sympy as sym # For Symbolic Calculus
from sympy import exp, sin, cos, Heaviside # Define symbolic functions of interest
# Import Laplace and Inverse Laplace Transforms
# For this step we use symbolic calculus
# h(t)=derivative of y_S
# we first define the variables we will need
h,y_S, t,s,alpha = sym.symbols('h y_S t s alpha')
# We give some additional (helpful info)
y_S=1/5-1/5*sym.exp(-t)*(sym.cos(2*t)+1/2*sym.sin(2*t)); print(y_S)
# Hence we compute h(t) by differentiation
h=sym.simplify(sym.diff(y_S,t)); print(h)


####### OPTIONAL ###########

## Numerical Double Check using numerical integration via odeint
from scipy.integrate import odeint
# The numerical solver demands for setting the system in state-space form
def Sys1(X,t_kk): 
    # this system is for u_1
    x1=X[0]
    x2=X[1]
    x1_d=x2
    x2_d=-2*x2-5*x1+np.exp(-t_kk)
    return [x1_d,x2_d]

X0=[0,0]; # Initial condition 
t_k=np.linspace(0,T,100) # Discretization for numerical solver
t_k_a=np.linspace(0,T,500) # Discretization for analytical sol

sol=odeint(Sys1,X0,t_k)
theta=sol[:,0]
theta_a=-0.25*(np.cos(2*t_k_a)-1)*np.exp(-t_k_a)

#####################################################################
# Finally we compute the convolution numerically.
######################################################################
# This is the case at Fs=10Hz#############################
fs=10; dt_10=1/fs # Case at Fs=10Hz
t_k_10=np.arange(0,T,dt_10)
y_k_Conv_10=np.zeros(t_k_10.shape) # Initialize output y_k convolution
# Define constants of the sequence
a=np.exp(-dt_10); b=2*dt_10

for k in range(0,len(y_k_Conv_10)): # Loop to fill the vector
 for l in range(0,k):
  y_k_Conv_10[k]=(0.5*a**(k)*np.sin(b*l)+y_k_Conv_10[k])
#
## Check that this is NOT equivalent to the convolution:
#k_I=np.arange(0,len(t_k),1)  
#h_I=0.5*a**k_I*np.sin(b*k_I)
#u_I=a**(k_I)
#y_k_Conv_10_2=np.convolve(u_I,h_I,mode='full')
#
#plt.plot(t_k_10,y_k_Conv_10)
#plt.plot(t_k_10,y_k_Conv_10_2[0:len(t_k_10)])
# The reason is that h is of infinite duration.

# This is the case at Fs=3 Hz #############################
fs=3; dt_3=1/fs # Case at Fs=3Hz
t_k_3=np.arange(0,T+dt_3,dt_3)
y_k_Conv_3=np.zeros(t_k_3.shape) # Initialize output y_k convolution
# Define constants of the sequence
a=np.exp(-dt_3); b=2*dt_3

for k in range(0,len(y_k_Conv_3)): # Loop to fill the vector
 for l in range(0,k):
  y_k_Conv_3[k]=(0.5*a**(k)*np.sin(b*l)+y_k_Conv_3[k])


# Show the numerical results and the analytical one
fig, ax = plt.subplots(figsize=(5, 3)) # This creates the figure
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
# Or you can plot it as streamlines
plt.plot(t_k_a,theta_a,label='y$(t_k)$')
#plt.plot(t_k,theta,'ro') # Uncomment to compare also with the numerical results
plt.plot(t_k_10,y_k_Conv_10*dt_10,'ko',label='$y_{10}[k]$')
plt.plot(t_k_3,y_k_Conv_3*dt_3,'rs',label='$y_{3}[k]$')
ax.set_xlabel('$t_k [s]$',fontsize=16)
ax.set_xlim(0,5)
ax.legend(fontsize=16)
#ax.set_title('$y[k] \, vs \, y(t_k)$',fontsize=14)

plt.tight_layout()
Name='Ex2b.pdf'
plt.savefig(Name, dpi=200) 
#shutil.copyfile(Name,'../Figures/'+Name) #  copy file to latex directory




