# -*- coding: utf-8 -*-
"""
Created on Sat Jan 25 17:24:33 2020

@author: mendez
"""

# Design a filter bank using FIR or IIR.
import numpy as np
import matplotlib.pyplot as plt  # Matplotlib is for plotting.
import shutil # To copy the figures in the Latex folder

## Step 0: Construct the signal with its scales:
n_t=4096 # Number of point
k=np.arange(0,n_t,1)
f_s=1000; dt=1/f_s # Compute Delta t
t_k=k*dt # Time domain axis
# Mode 1
u_1=2*np.sin(2*np.pi*20*t_k)*np.exp(-(t_k-1)**2/(0.05))\
# Mode 2
u_2=1*np.sin(2*np.pi*90*t_k)*np.exp(-(t_k-2.2)**2/(0.05))\
# Mode 3
u_3=1*np.sin(2*np.pi*1*t_k)\

u=u_1+u_2+u_3+np.random.normal(0, 0.2, size=n_t) 

# Figure 1: plot in the time domain

fig, ax = plt.subplots(figsize=(5, 3)) # This creates the figure
plt.rc('text', usetex=True)      
plt.rc('font', family='serif')
plt.rc('xtick',labelsize=13)
plt.rc('ytick',labelsize=13)
plt.plot(t_k,u)
# This is due to the phase delay.
#ax.set_aspect('equal') # Set equal aspect ratio
plt.xlabel('$t_k[s]$',fontsize=14)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
Name='Ex_5_time.pdf'
plt.tight_layout()
plt.savefig(Name, dpi=200) 
#shutil.copyfile(Name,'../Figures/'+Name) #  copy file to latex directory

# Compute also the fft
U_n=np.fft.fft(u)/np.sqrt(n_t)
Freqs=np.fft.fftfreq(n_t)*f_s

# Figure 2: Plot in the frequency domain
fig, ax = plt.subplots(figsize=(5, 3)) # This creates the figure
plt.plot(np.fft.fftshift(Freqs),np.absolute(np.fft.fftshift(U_n)),'k-')
plt.xlabel('$f_n[Hz]$',fontsize=14)
plt.xscale('log'); plt.yscale('log')
ax.set_xlim(0,f_s/2)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
Name='Ex_5_freq.pdf'
plt.tight_layout()
plt.savefig(Name, dpi=200) 
#shutil.copyfile(Name,'../Figures/'+Name) #  copy file to latex directory



# Compute low pass filter kernel by hand
f_c=10; N=511; theta_c=2*np.pi*f_c/f_s; alpha=(N-1)/2;
k=np.arange(0,N,1)
h_id=(np.sin((k-alpha)*theta_c))/(np.pi*(k-alpha))
# Replace the NAN:
h_id[np.isnan(h_id)]=theta_c/np.pi
#h_id=np.sinc((k-alpha)*theta_c)
# Construct the Hamming window
w=0.54-0.46*np.cos(2*np.pi*k/(N-1))
# The final kernel is
h_L=h_id*w
# Normalize so that the sum is one:
h_Ln=h_L/np.sum(h_L)

from scipy import signal
# use Python's function
h_LP=signal.firwin(N, f_c, width=None, window='hamming', pass_zero=True, scale=True, nyq=None, fs=f_s)
print(h_Ln)
print(h_LP)

import time # Usefull for timing the code
# Now perform the filtering in 4 ways
# Method 1: Convolution
# Start Timing
Begin=time.time() 
# Initialize h_L
h_L_c=np.zeros(u.shape);h_L_c[0:N]=h_LP 
# Direct convolution
u_LC=signal.convolve(u,h_Ln,mode='full')
# Take only the first part
u_L1=u_LC[0:len(u)]
# Stop timing
T=(time.time()-Begin); 
print('conv takes '+str(round(T,2))+' s') 

# Method 2: Convolution via FFT
Begin=time.time() 
u_LC=signal.fftconvolve(u,h_Ln,mode='full')
u_L2=u_LC[0:len(u)]
T=(time.time()-Begin); 
print('fft conv takes '+str(round(T,2))+' s') 

# Method 3: Convolution via FFT (manual)
Begin=time.time() 
H_L=np.fft.fft(h_L,len(t_k)) # Compute frequency transfer function
U=np.fft.fft(u,len(t_k))
u_L3 = np.real(np.fft.ifft(H_L*U))
T=(time.time()-Begin); 
print('Manual fft conv takes '+str(round(T,2))+' s') 

# Method 4: Convolution via recursive Implementation
Begin=time.time() 
u_L4 = signal.lfilter(h_L,1,u) 
T=(time.time()-Begin);
print('Recursive Eq takes '+str(round(T,2))+' s') 

## MRA part 3
F_V=np.array([10,70,110,300])
def MRA_SISO(u,F_V,f_s,N_O):
  """
    This function computes the MRA of a signal u
    using Hamming Windows   
    :param u: Input Signal
    :param F_V: Frequency Splitting Vectors (see notes) 
    :param f_s: Sampling Frequency
    :param N_O: Order of the Filter   
    :return: U_MRA, n_M scale partitions of the signal
             H_MRA  n_M scale Amplitude Responses Functions
  """
  # Get number of scales and number of points
  n_M=len(F_V)+1; n_t=len(u)
  # Initialize the output
  U_MRA=np.zeros((n_t,n_M))
  # Initialize the Transfer Function Matrix
  H_MRA=np.zeros((512,n_M))  
  # Loop from highest frequency to lowest:
  F_V_o=-np.sort(-F_V)
  # Loop over the scales
  for m in range(0,n_M):
   if m==0:
    #Create first low pass kernel     
    h=signal.firwin(N, F_V_o[m], pass_zero=True, fs=f_s)
    # Get frequency response (for checking)
    w,H_L=signal.freqz(h) 
    # This is the first large scale
    u_L=signal.fftconvolve(u,h,'same'); u_H=u-u_L; 
    U_MRA[:,m]=u_H
    H_MRA[:,m]=1-abs(H_L)
    print('Scale '+str(m),'Computed')
   elif m>0 and m<n_M-1:
    #Create mth low pass kernel     
    h=signal.firwin(N, F_V_o[m], pass_zero=True, fs=f_s)
    w,H_L_new=signal.freqz(h) 
    # Low-pass filter m+1
    u_L_new=signal.fftconvolve(u,h,'same')
    # Band Pass filtered and store
    u_H=u_L-u_L_new; U_MRA[:,m]=u_H
    u_L=u_L_new
    # Get Frequency Transf Function and store
    H_MRA[:,m]=abs(H_L)-abs(H_L_new); H_L=H_L_new
    print('Scale '+str(m),'Computed')
   else: 
    U_MRA[:,m]=u_L_new
    # Get Frequency Transf Function and store
    H_MRA[:,m]=abs(H_L); 
    print('Scale '+str(m),'Computed')
    print('Done!')
  return U_MRA,H_MRA
    
# Test Function:
U_MRA,H_MRA=MRA_SISO(u,F_V,f_s,511)  
fig, ax = plt.subplots(figsize=(5, 3)) # This creates the figure
plt.plot(np.fft.fftshift(Freqs),\
         np.absolute(np.fft.fftshift(U_n))/np.abs(U_n).max(),'k-')
for m in range(0,len(F_V)+1):
 plt.plot(H_MRA[:,m])    
plt.xlabel('$f_n[Hz]$',fontsize=16)
plt.xscale('log'); 
#plt.yscale('log')
ax.set_xlim(0.2,f_s/2)
ax.set_ylim(10e-4,1)
ax.spines['top'].set_visible(False)
ax.spines['right'].set_visible(False)
Name='MRA_SCALES.pdf'
plt.tight_layout()
plt.savefig(Name, dpi=200) 
#shutil.copyfile(Name,'../Figures/'+Name) #  copy file to latex directory
    
    
# Plot Portions
fig, ax = plt.subplots(figsize=(10, 4)) # This creates the figure
#plt.subplot(5,1,1)
#plt.plot(t_k,u)
#plt.title('$u(t_k)$',fontsize=16)
plt.subplot(2,2,1)
plt.plot(t_k,U_MRA[:,4])
plt.title('$s_1(t_k)$',fontsize=16)
plt.subplot(2,2,2)
plt.plot(t_k,U_MRA[:,3])
plt.title('$s_2(t_k)$',fontsize=16)
plt.subplot(2,2,3)
plt.plot(t_k,U_MRA[:,2])
plt.title('$s_3(t_k)$',fontsize=16)
plt.subplot(2,2,4)
plt.plot(t_k,U_MRA[:,1])
plt.title('$s_4(t_k)$',fontsize=16)
plt.xlabel('$t_k[s]$')
Name='MRA_Components.pdf'
plt.tight_layout()
plt.savefig(Name, dpi=200) 
#shutil.copyfile(Name,'../Figures/'+Name) #  copy file to latex directory




